package com.tourism.webadmin.back.dto;

import com.tourism.common.core.validator.UpdateGroup;
import com.tourism.common.core.validator.ConstDictRef;
import com.tourism.webadmin.back.model.constant.Enable;

import io.swagger.v3.oas.annotations.media.Schema;
import lombok.Data;

import jakarta.validation.constraints.*;

/**
 * 游记话题Dto对象。
 *
 * @author 吃饭睡觉
 * @date 2024-09-06
 */
@Schema(description = "游记话题Dto对象")
@Data
public class TourTravelNotesTopicDto {

    /**
     * 主键id。
     */
    @Schema(description = "主键id。", requiredMode = Schema.RequiredMode.REQUIRED)
    @NotNull(message = "数据验证失败，主键id不能为空！", groups = {UpdateGroup.class})
    private Long id;

    /**
     * 话题名称。
     * NOTE: 可支持等于操作符的列表数据过滤。
     */
    @Schema(description = "话题名称。可支持等于操作符的列表数据过滤。")
    private String name;

    /**
     * 是否启用，0否，1是。
     * NOTE: 可支持等于操作符的列表数据过滤。
     */
    @Schema(description = "是否启用，0否，1是。可支持等于操作符的列表数据过滤。")
    @ConstDictRef(constDictClass = Enable.class, message = "数据验证失败，是否启用，0否，1是为无效值！")
    private Integer enable;

    /**
     * 热度值。
     * NOTE: 可支持等于操作符的列表数据过滤。
     */
    @Schema(description = "热度值。可支持等于操作符的列表数据过滤。")
    private Integer hotValue;

    /**
     * hotValue 范围过滤起始值(>=)。
     * NOTE: 可支持范围操作符的列表数据过滤。
     */
    @Schema(description = "hotValue 范围过滤起始值(>=)。可支持范围操作符的列表数据过滤。")
    private Integer hotValueStart;

    /**
     * hotValue 范围过滤结束值(<=)。
     * NOTE: 可支持范围操作符的列表数据过滤。
     */
    @Schema(description = "hotValue 范围过滤结束值(<=)。可支持范围操作符的列表数据过滤。")
    private Integer hotValueEnd;
}
