package com.tourism.webadmin.back.model;

import com.baomidou.mybatisplus.annotation.*;
import com.tourism.webadmin.upms.model.SysUser;
import com.tourism.webadmin.back.model.constant.IsTrue;
import com.tourism.common.core.util.MyCommonUtil;
import com.tourism.common.core.upload.UploadStoreTypeEnum;
import com.tourism.common.core.annotation.*;
import com.tourism.common.core.base.model.BaseModel;
import lombok.Data;
import lombok.EqualsAndHashCode;

import java.util.Map;
import java.util.List;

/**
 * 聊天群聊管理实体对象。
 *
 * @author 吃饭睡觉
 * @date 2024-09-06
 */
@Data
@EqualsAndHashCode(callSuper = true)
@TableName(value = "tour_im_group")
public class TourImGroup extends BaseModel {

    /**
     * id。
     */
    @TableId(value = "id")
    private Long id;

    /**
     * 群聊所属类型id。
     */
    @TableField(value = "belong_type_id")
    private Long belongTypeId;

    /**
     * 群主id。
     */
    @TableField(value = "leader_id")
    private Long leaderId;

    /**
     * 群聊名称。
     */
    @TableField(value = "group_name")
    private String groupName;

    /**
     * 群聊头像。
     */
    @UploadFlagColumn(storeType = UploadStoreTypeEnum.HUAWEI_OBS_SYSTEM)
    @TableField(value = "group_avatar")
    private String groupAvatar;

    /**
     * 群聊描述。
     */
    @TableField(value = "description")
    private String description;

    /**
     * 是否公开展示 0隐藏 1公开。
     */
    @TableField(value = "is_public")
    private Integer isPublic;

    /**
     * 是否开启群聊邀请确认 0不开启 1开启。
     */
    @TableField(value = "is_need_confirm")
    private Integer isNeedConfirm;

    /**
     * 通知分类 1单聊 2群聊 3系统
     */
    @TableField(value = "notice_type")
    private Integer noticeType;

    /**
     * 是否封禁 0封禁 1正常。
     */
    @TableField(value = "banned_status")
    private Integer bannedStatus;

    /**
     * 热度值。
     */
    @TableField(value = "hot_value")
    private Integer hotValue;

    /**
     * 逻辑删除标记字段(1: 正常 -1: 已删除)。
     */
//    @TableLogic
    @TableField(value = "data_state")
    private Integer dataState;

    /**
     * hotValue 范围过滤起始值(>=)。
     */
    @TableField(exist = false)
    private Integer hotValueStart;

    /**
     * hotValue 范围过滤结束值(<=)。
     */
    @TableField(exist = false)
    private Integer hotValueEnd;

    /**
     * group_name / description LIKE搜索字符串。
     */
    @TableField(exist = false)
    private String searchString;

    public void setSearchString(String searchString) {
        this.searchString = MyCommonUtil.replaceSqlWildcard(searchString);
    }

    /**
     * TourImGroupInvitation 的一对多关联表数据对象。
     * 通常在一对多的关联中，我们基于从表数据过滤主表数据，此时需要先对从表数据进行嵌套子查询过滤，并将从表过滤数据列表集成到该字段。
     */
    @RelationOneToMany(
            masterIdField = "id",
            slaveModelClass = TourImGroupInvitation.class,
            slaveIdField = "groupId")
    @TableField(exist = false)
    private List<TourImGroupInvitation> tourImGroupInvitationList;

    /**
     * TourImMember 的一对多关联表数据对象。
     * 通常在一对多的关联中，我们基于从表数据过滤主表数据，此时需要先对从表数据进行嵌套子查询过滤，并将从表过滤数据列表集成到该字段。
     */
    @RelationOneToMany(
            masterIdField = "id",
            slaveModelClass = TourImMember.class,
            slaveIdField = "groupId")
    @TableField(exist = false)
    private List<TourImMember> tourImMemberList;

    /**
     * TourImMessage 的一对多关联表数据对象。
     * 通常在一对多的关联中，我们基于从表数据过滤主表数据，此时需要先对从表数据进行嵌套子查询过滤，并将从表过滤数据列表集成到该字段。
     */
    @RelationOneToMany(
            masterIdField = "id",
            slaveModelClass = TourImMessage.class,
            slaveIdField = "groupId")
    @TableField(exist = false)
    private List<TourImMessage> tourImMessageList;

    @RelationDict(
            masterIdField = "belongTypeId",
            slaveModelClass = TourImGroupType.class,
            slaveIdField = "id",
            slaveNameField = "typeName")
    @TableField(exist = false)
    private Map<String, Object> belongTypeIdDictMap;

    @RelationDict(
            masterIdField = "leaderId",
            slaveModelClass = TourUser.class,
            slaveIdField = "userId",
            slaveNameField = "showName")
    @TableField(exist = false)
    private Map<String, Object> leaderIdDictMap;

    @RelationConstDict(
            masterIdField = "isPublic",
            constantDictClass = IsTrue.class)
    @TableField(exist = false)
    private Map<String, Object> isPublicDictMap;

    @RelationConstDict(
            masterIdField = "isNeedConfirm",
            constantDictClass = IsTrue.class)
    @TableField(exist = false)
    private Map<String, Object> isNeedConfirmDictMap;

    @RelationConstDict(
            masterIdField = "bannedStatus",
            constantDictClass = IsTrue.class)
    @TableField(exist = false)
    private Map<String, Object> bannedStatusDictMap;

    @RelationGlobalDict(
            masterIdField = "noticeType",
            dictCode = "TourImSingleOrGroup")
    @TableField(exist = false)
    private Map<String, Object> noticeTypeDictMap;
}
