package com.tourism.webadmin.back.service;

import com.tourism.webadmin.back.model.*;
import com.tourism.common.core.base.service.IBaseService;

import java.util.*;

/**
 * 游记话题数据操作服务接口。
 *
 * @author 吃饭睡觉
 * @date 2024-09-06
 */
public interface TourTravelNotesTopicService extends IBaseService<TourTravelNotesTopic, Long> {

    /**
     * 保存新增对象。
     *
     * @param tourTravelNotesTopic 新增对象。
     * @return 返回新增对象。
     */
    TourTravelNotesTopic saveNew(TourTravelNotesTopic tourTravelNotesTopic);

    /**
     * 利用数据库的insertList语法，批量插入对象列表。
     *
     * @param tourTravelNotesTopicList 新增对象列表。
     */
    void saveNewBatch(List<TourTravelNotesTopic> tourTravelNotesTopicList);

    /**
     * 更新数据对象。
     *
     * @param tourTravelNotesTopic         更新的对象。
     * @param originalTourTravelNotesTopic 原有数据对象。
     * @return 成功返回true，否则false。
     */
    boolean update(TourTravelNotesTopic tourTravelNotesTopic, TourTravelNotesTopic originalTourTravelNotesTopic);

    /**
     * 删除指定数据。
     *
     * @param id 主键Id。
     * @return 成功返回true，否则false。
     */
    boolean remove(Long id);

    /**
     * 获取单表查询结果。由于没有关联数据查询，因此在仅仅获取单表数据的场景下，效率更高。
     * 如果需要同时获取关联数据，请移步(getTourTravelNotesTopicListWithRelation)方法。
     *
     * @param filter  过滤对象。
     * @param orderBy 排序参数。
     * @return 查询结果集。
     */
    List<TourTravelNotesTopic> getTourTravelNotesTopicList(TourTravelNotesTopic filter, String orderBy);

    /**
     * 获取主表的查询结果，以及主表关联的字典数据和一对一从表数据，以及一对一从表的字典数据。
     * 该查询会涉及到一对一从表的关联过滤，或一对多从表的嵌套关联过滤，因此性能不如单表过滤。
     * 如果仅仅需要获取主表数据，请移步(getTourTravelNotesTopicList)，以便获取更好的查询性能。
     *
     * @param filter 主表过滤对象。
     * @param orderBy 排序参数。
     * @return 查询结果集。
     */
    List<TourTravelNotesTopic> getTourTravelNotesTopicListWithRelation(TourTravelNotesTopic filter, String orderBy);

    /**
     * 根据游记话题名称查询话题浏览量
     *
     * @return 查询结果集。
     */
    Integer getViewCountTopicByName(String name,String titleName);


    /**
     * 根据游记话题名称查询话题浏览量
     *
     * @return 查询结果集。
     */
    Map<Long, Integer> getViewCountListTopicByName(List<Long> idList);

    /**
     * 根据游记话题名称查询话题评论数
     *
     * @return 查询结果集。
     */
    Integer getCommentCountTopicByName(String name,String titleName);

    /**
     * 根据话题名称查询游记数量
     *
     * @return 查询结果集。
     */
    Integer getCountTravelNotesByName(String name,String titleName);

    /**
     * 根据游记话题名称查询关联的游记id
     *
     * @return 查询结果集。
     */
    List<TourismProjectTravelNotesWrite> getTravelNotesByTopicName(String name,String orderBy);
}
