package com.tourism.webadmin.back.service.impl;

import cn.hutool.core.collection.CollUtil;
import com.baomidou.mybatisplus.core.conditions.query.*;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.tourism.webadmin.back.service.*;
import com.tourism.webadmin.back.dao.*;
import com.tourism.webadmin.back.model.*;
import com.tourism.webadmin.config.DataSourceType;
import com.tourism.webadmin.upms.service.SysUserService;
import com.tourism.common.core.annotation.MyDataSource;
import com.tourism.common.core.base.dao.BaseDaoMapper;
import com.tourism.common.core.constant.GlobalDeletedFlag;
import com.tourism.common.core.object.MyRelationParam;
import com.tourism.common.core.object.CallResult;
import com.tourism.common.core.base.service.BaseService;
import com.tourism.common.core.util.MyModelUtil;
import com.tourism.common.sequence.wrapper.IdGeneratorWrapper;
import com.tourism.common.dict.service.GlobalDictService;
import com.github.pagehelper.Page;
import lombok.extern.slf4j.Slf4j;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.*;

/**
 * 聊天去聊邀请数据操作服务类。
 *
 * @author 吃饭睡觉
 * @date 2024-09-06
 */
@Slf4j
@MyDataSource(DataSourceType.MAIN)
@Service("tourImGroupInvitationService")
public class TourImGroupInvitationServiceImpl extends BaseService<TourImGroupInvitation, Long> implements TourImGroupInvitationService {

    @Autowired
    private IdGeneratorWrapper idGenerator;
    @Autowired
    private TourImGroupInvitationMapper tourImGroupInvitationMapper;
    @Autowired
    private TourImGroupService tourImGroupService;
    @Autowired
    private SysUserService sysUserService;
    @Autowired
    private GlobalDictService globalDictService;

    /**
     * 返回当前Service的主表Mapper对象。
     *
     * @return 主表Mapper对象。
     */
    @Override
    protected BaseDaoMapper<TourImGroupInvitation> mapper() {
        return tourImGroupInvitationMapper;
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public TourImGroupInvitation saveNew(TourImGroupInvitation tourImGroupInvitation) {
        tourImGroupInvitationMapper.insert(this.buildDefaultValue(tourImGroupInvitation));
        return tourImGroupInvitation;
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public void saveNewBatch(List<TourImGroupInvitation> tourImGroupInvitationList) {
        if (CollUtil.isNotEmpty(tourImGroupInvitationList)) {
            tourImGroupInvitationList.forEach(this::buildDefaultValue);
            tourImGroupInvitationMapper.insertList(tourImGroupInvitationList);
        }
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public void saveNewBatch(List<TourImGroupInvitation> tourImGroupInvitationList, int batchSize) {
        if (CollUtil.isEmpty(tourImGroupInvitationList)) {
            return;
        }
        if (batchSize <= 0) {
            batchSize = 10000;
        }
        int start = 0;
        do {
            int end = Math.min(tourImGroupInvitationList.size(), start + batchSize);
            List<TourImGroupInvitation> subList = tourImGroupInvitationList.subList(start, end);
            // 如果数据量过大，同时当前表中存在createTime或updateTime等字段，可以考虑在外部创建一次 new Date()，
            // 然后传入buildDefaultValue，这样可以减少对象的创建次数，降低GC，提升效率。橙单之所以没有这样生成，是因为
            // 有些业务场景下需要按照这两个日期字段排序，因此我们只是在这里给出优化建议。
            subList.forEach(this::buildDefaultValue);
            tourImGroupInvitationMapper.insertList(subList);
            if (end == tourImGroupInvitationList.size()) {
                break;
            }
            start += batchSize;
        } while (true);
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public boolean update(TourImGroupInvitation tourImGroupInvitation, TourImGroupInvitation originalTourImGroupInvitation) {
        MyModelUtil.fillCommonsForUpdate(tourImGroupInvitation, originalTourImGroupInvitation);
        // 这里重点提示，在执行主表数据更新之前，如果有哪些字段不支持修改操作，请用原有数据对象字段替换当前数据字段。
        UpdateWrapper<TourImGroupInvitation> uw = this.createUpdateQueryForNullValue(tourImGroupInvitation, tourImGroupInvitation.getId());
        return tourImGroupInvitationMapper.update(tourImGroupInvitation, uw) == 1;
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public boolean remove(Long id) {
        return tourImGroupInvitationMapper.deleteById(id) == 1;
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public int removeByGroupId(Long groupId) {
        TourImGroupInvitation deletedObject = new TourImGroupInvitation();
        deletedObject.setGroupId(groupId);
        return tourImGroupInvitationMapper.delete(new QueryWrapper<>(deletedObject));
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public void updateBatchByGroupId(Long groupId, List<TourImGroupInvitation> dataList) {
        this.updateBatchOneToManyRelation("groupId", groupId,
                null, null, dataList, this::saveNewBatch);
    }

    @Override
    public List<TourImGroupInvitation> getTourImGroupInvitationList(TourImGroupInvitation filter, String orderBy) {
        return tourImGroupInvitationMapper.getTourImGroupInvitationList(filter, orderBy);
    }

    @Override
    public List<TourImGroupInvitation> getTourImGroupInvitationListWithRelation(TourImGroupInvitation filter, String orderBy) {
        List<TourImGroupInvitation> resultList = tourImGroupInvitationMapper.getTourImGroupInvitationList(filter, orderBy);
        // 在缺省生成的代码中，如果查询结果resultList不是Page对象，说明没有分页，那么就很可能是数据导出接口调用了当前方法。
        // 为了避免一次性的大量数据关联，规避因此而造成的系统运行性能冲击，这里手动进行了分批次读取，开发者可按需修改该值。
        int batchSize = resultList instanceof Page ? 0 : 1000;
        this.buildRelationForDataList(resultList, MyRelationParam.normal(), batchSize);
        return resultList;
    }

    @Override
    public CallResult verifyImportList(List<TourImGroupInvitation> dataList, Set<String> ignoreFieldSet) {
        CallResult callResult;
        if (!CollUtil.contains(ignoreFieldSet, "status")) {
            callResult = verifyImportForGlobalDict(dataList, "statusDictMap", TourImGroupInvitation::getStatus);
            if (!callResult.isSuccess()) {
                return callResult;
            }
        }
        if (!CollUtil.contains(ignoreFieldSet, "groupId")) {
            callResult = verifyImportForDatasourceDict(dataList, "groupIdDictMap", TourImGroupInvitation::getGroupId);
            if (!callResult.isSuccess()) {
                return callResult;
            }
        }
        if (!CollUtil.contains(ignoreFieldSet, "inviter")) {
            callResult = verifyImportForDatasourceDict(dataList, "inviterDictMap", TourImGroupInvitation::getInviter);
            if (!callResult.isSuccess()) {
                return callResult;
            }
        }
        if (!CollUtil.contains(ignoreFieldSet, "invitee")) {
            callResult = verifyImportForDatasourceDict(dataList, "inviteeDictMap", TourImGroupInvitation::getInvitee);
            if (!callResult.isSuccess()) {
                return callResult;
            }
        }
        return CallResult.ok();
    }

    @Override
    public CallResult verifyRelatedData(TourImGroupInvitation tourImGroupInvitation, TourImGroupInvitation originalTourImGroupInvitation) {
        String errorMessageFormat = "数据验证失败，关联的%s并不存在，请刷新后重试！";
        //这里是基于字典的验证。
        if (this.needToVerify(tourImGroupInvitation, originalTourImGroupInvitation, TourImGroupInvitation::getGroupId)
                && !tourImGroupService.existId(tourImGroupInvitation.getGroupId())) {
            return CallResult.error(String.format(errorMessageFormat, "聊天群组id"));
        }
        //这里是基于字典的验证。
        if (this.needToVerify(tourImGroupInvitation, originalTourImGroupInvitation, TourImGroupInvitation::getInviter)
                && !sysUserService.existId(tourImGroupInvitation.getInviter())) {
            return CallResult.error(String.format(errorMessageFormat, "邀请人id"));
        }
        //这里是基于字典的验证。
        if (this.needToVerify(tourImGroupInvitation, originalTourImGroupInvitation, TourImGroupInvitation::getInvitee)
                && !sysUserService.existId(tourImGroupInvitation.getInvitee())) {
            return CallResult.error(String.format(errorMessageFormat, "被邀请人id"));
        }
        //这里是基于字典的验证。
        if (this.needToVerify(tourImGroupInvitation, originalTourImGroupInvitation, TourImGroupInvitation::getStatus)
                && !globalDictService.existDictItemFromCache("GroupInvitationStatus", tourImGroupInvitation.getStatus())) {
            return CallResult.error(String.format(errorMessageFormat, "邀请状态 1待确认 2同意 3不同意"));
        }
        return CallResult.ok();
    }

    private TourImGroupInvitation buildDefaultValue(TourImGroupInvitation tourImGroupInvitation) {
        if (tourImGroupInvitation.getId() == null) {
            tourImGroupInvitation.setId(idGenerator.nextLongId());
        }
        MyModelUtil.fillCommonsForInsert(tourImGroupInvitation);
        tourImGroupInvitation.setDataState(GlobalDeletedFlag.NORMAL);
        return tourImGroupInvitation;
    }
}
