package com.tourism.webadmin.back.service.impl;

import cn.hutool.core.collection.CollUtil;

import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.tourism.webadmin.back.dao.TourTourismProjectSearchRecordsMapper;
import com.tourism.webadmin.back.model.TourTourismProjectSearchRecords;
import com.tourism.webadmin.back.service.TourTourismProjectSearchRecordsService;
import com.tourism.common.core.base.dao.BaseDaoMapper;
import com.tourism.common.core.constant.GlobalDeletedFlag;
import com.tourism.common.core.object.MyRelationParam;
import com.tourism.common.core.base.service.BaseService;
import com.tourism.common.core.util.MyModelUtil;
import com.tourism.common.sequence.wrapper.IdGeneratorWrapper;
import com.github.pagehelper.Page;
import lombok.extern.slf4j.Slf4j;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.*;

/**
 * 游记列表搜索数据操作服务类。
 *
 * @author 奇幻之旅
 * @date 2024-09-06
 */
@Slf4j
@Service("tourTourismProjectRecordsService")
public class TourTourismProjectRecordsServiceImpl extends BaseService<TourTourismProjectSearchRecords, Long> implements TourTourismProjectSearchRecordsService {

    @Autowired
    private IdGeneratorWrapper idGenerator;
    @Autowired
    private TourTourismProjectSearchRecordsMapper tourTourismSearchRecordsMapper;

    /**
     * 返回当前Service的主表Mapper对象。
     *
     * @return 主表Mapper对象。
     */
    @Override
    protected BaseDaoMapper<TourTourismProjectSearchRecords> mapper() {
        return tourTourismSearchRecordsMapper;
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public TourTourismProjectSearchRecords saveNew(TourTourismProjectSearchRecords tourTourismSearchRecords) {
        tourTourismSearchRecordsMapper.insert(this.buildDefaultValue(tourTourismSearchRecords));
        return tourTourismSearchRecords;
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public void saveNewBatch(List<TourTourismProjectSearchRecords> tourTourismSearchRecordsList) {
        if (CollUtil.isNotEmpty(tourTourismSearchRecordsList)) {
            tourTourismSearchRecordsList.forEach(this::buildDefaultValue);
            tourTourismSearchRecordsMapper.insertList(tourTourismSearchRecordsList);
        }
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public boolean update(TourTourismProjectSearchRecords tourTourismSearchRecords, TourTourismProjectSearchRecords originalTourTourismSearchRecords) {
        MyModelUtil.fillCommonsForUpdate(tourTourismSearchRecords, originalTourTourismSearchRecords);
        // 这里重点提示，在执行主表数据更新之前，如果有哪些字段不支持修改操作，请用原有数据对象字段替换当前数据字段。
        UpdateWrapper<TourTourismProjectSearchRecords> uw = this.createUpdateQueryForNullValue(tourTourismSearchRecords, tourTourismSearchRecords.getId());
        return tourTourismSearchRecordsMapper.update(tourTourismSearchRecords, uw) == 1;
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public boolean remove(Long id) {
        return tourTourismSearchRecordsMapper.deleteById(id) == 1;
    }

    @Override
    public List<TourTourismProjectSearchRecords> getTourTourismSearchRecordsList(TourTourismProjectSearchRecords filter, String orderBy) {
        return tourTourismSearchRecordsMapper.getTourTourismSearchRecordsList(filter, orderBy);
    }

    @Override
    public List<TourTourismProjectSearchRecords> getTourTourismSearchRecordsListWithRelation(TourTourismProjectSearchRecords filter, String orderBy) {
        List<TourTourismProjectSearchRecords> resultList = tourTourismSearchRecordsMapper.getTourTourismSearchRecordsList(filter, orderBy);
        // 在缺省生成的代码中，如果查询结果resultList不是Page对象，说明没有分页，那么就很可能是数据导出接口调用了当前方法。
        // 为了避免一次性的大量数据关联，规避因此而造成的系统运行性能冲击，这里手动进行了分批次读取，开发者可按需修改该值。
        int batchSize = resultList instanceof Page ? 0 : 1000;
        this.buildRelationForDataList(resultList, MyRelationParam.normal(), batchSize);
        return resultList;
    }

    private TourTourismProjectSearchRecords buildDefaultValue(TourTourismProjectSearchRecords tourTourismSearchRecords) {
        if (tourTourismSearchRecords.getId() == null) {
            tourTourismSearchRecords.setId(idGenerator.nextLongId());
        }
        MyModelUtil.fillCommonsForInsert(tourTourismSearchRecords);
        tourTourismSearchRecords.setDataState(GlobalDeletedFlag.NORMAL);
        return tourTourismSearchRecords;
    }
}
