package com.tourism.webadmin.back.service.impl;

import cn.hutool.core.collection.CollUtil;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.tourism.webadmin.back.service.*;
import com.tourism.webadmin.back.dao.*;
import com.tourism.webadmin.back.model.*;
import com.tourism.webadmin.config.DataSourceType;
import com.tourism.common.core.annotation.MyDataSource;
import com.tourism.common.core.base.dao.BaseDaoMapper;
import com.tourism.common.core.object.MyRelationParam;
import com.tourism.common.core.base.service.BaseService;
import com.tourism.common.sequence.wrapper.IdGeneratorWrapper;
import com.github.pagehelper.Page;
import lombok.extern.slf4j.Slf4j;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.*;

/**
 * 游记与话题关联数据操作服务类。
 *
 * @author 吃饭睡觉
 * @date 2024-09-06
 */
@Slf4j
@Service("tourTravelNotesTopicRelationService")
public class TourTravelNotesTopicRelationServiceImpl extends BaseService<TourTravelNotesTopicRelation, Long> implements TourTravelNotesTopicRelationService {

    @Autowired
    private IdGeneratorWrapper idGenerator;
    @Autowired
    private TourTravelNotesTopicRelationMapper tourTravelNotesTopicRelationMapper;

    /**
     * 返回当前Service的主表Mapper对象。
     *
     * @return 主表Mapper对象。
     */
    @Override
    protected BaseDaoMapper<TourTravelNotesTopicRelation> mapper() {
        return tourTravelNotesTopicRelationMapper;
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public TourTravelNotesTopicRelation saveNew(TourTravelNotesTopicRelation tourTravelNotesTopicRelation) {
        tourTravelNotesTopicRelationMapper.insert(this.buildDefaultValue(tourTravelNotesTopicRelation));
        return tourTravelNotesTopicRelation;
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public void saveNewBatch(List<TourTravelNotesTopicRelation> tourTravelNotesTopicRelationList) {
        if (CollUtil.isNotEmpty(tourTravelNotesTopicRelationList)) {
            tourTravelNotesTopicRelationList.forEach(this::buildDefaultValue);
            tourTravelNotesTopicRelationMapper.insertList(tourTravelNotesTopicRelationList);
        }
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public boolean update(TourTravelNotesTopicRelation tourTravelNotesTopicRelation, TourTravelNotesTopicRelation originalTourTravelNotesTopicRelation) {
        // 这里重点提示，在执行主表数据更新之前，如果有哪些字段不支持修改操作，请用原有数据对象字段替换当前数据字段。
        UpdateWrapper<TourTravelNotesTopicRelation> uw = this.createUpdateQueryForNullValue(tourTravelNotesTopicRelation, tourTravelNotesTopicRelation.getId());
        return tourTravelNotesTopicRelationMapper.update(tourTravelNotesTopicRelation, uw) == 1;
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public boolean remove(Long id) {
        return tourTravelNotesTopicRelationMapper.deleteById(id) == 1;
    }

    @Override
    public List<TourTravelNotesTopicRelation> getTourTravelNotesTopicRelationList(TourTravelNotesTopicRelation filter, String orderBy) {
        return tourTravelNotesTopicRelationMapper.getTourTravelNotesTopicRelationList(filter, orderBy);
    }

    @Override
    public List<TourTravelNotesTopicRelation> getTourTravelNotesTopicRelationListWithRelation(TourTravelNotesTopicRelation filter, String orderBy) {
        List<TourTravelNotesTopicRelation> resultList = tourTravelNotesTopicRelationMapper.getTourTravelNotesTopicRelationList(filter, orderBy);
        // 在缺省生成的代码中，如果查询结果resultList不是Page对象，说明没有分页，那么就很可能是数据导出接口调用了当前方法。
        // 为了避免一次性的大量数据关联，规避因此而造成的系统运行性能冲击，这里手动进行了分批次读取，开发者可按需修改该值。
        int batchSize = resultList instanceof Page ? 0 : 1000;
        this.buildRelationForDataList(resultList, MyRelationParam.normal(), batchSize);
        return resultList;
    }

    private TourTravelNotesTopicRelation buildDefaultValue(TourTravelNotesTopicRelation tourTravelNotesTopicRelation) {
        if (tourTravelNotesTopicRelation.getId() == null) {
            tourTravelNotesTopicRelation.setId(idGenerator.nextLongId());
        }
        return tourTravelNotesTopicRelation;
    }
}
