package com.tourism.webadmin.back.service.impl;

import cn.hutool.core.collection.CollUtil;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.tourism.webadmin.app.website.vo.NotesViewsVo;
import com.tourism.webadmin.back.service.*;
import com.tourism.webadmin.back.dao.*;
import com.tourism.webadmin.back.model.*;
import com.tourism.common.core.base.dao.BaseDaoMapper;
import com.tourism.common.core.constant.GlobalDeletedFlag;
import com.tourism.common.core.object.MyRelationParam;
import com.tourism.common.core.base.service.BaseService;
import com.tourism.common.core.util.MyModelUtil;
import com.tourism.common.sequence.wrapper.IdGeneratorWrapper;
import com.github.pagehelper.Page;
import lombok.extern.slf4j.Slf4j;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.*;

/**
 * 游记话题数据操作服务类。
 *
 * @author 吃饭睡觉
 * @date 2024-09-06
 */
@Slf4j
@Service("tourTravelNotesTopicService")
public class TourTravelNotesTopicServiceImpl extends BaseService<TourTravelNotesTopic, Long> implements TourTravelNotesTopicService {

    @Autowired
    private IdGeneratorWrapper idGenerator;
    @Autowired
    private TourTravelNotesTopicMapper tourTravelNotesTopicMapper;

    /**
     * 返回当前Service的主表Mapper对象。
     *
     * @return 主表Mapper对象。
     */
    @Override
    protected BaseDaoMapper<TourTravelNotesTopic> mapper() {
        return tourTravelNotesTopicMapper;
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public TourTravelNotesTopic saveNew(TourTravelNotesTopic tourTravelNotesTopic) {
        tourTravelNotesTopicMapper.insert(this.buildDefaultValue(tourTravelNotesTopic));
        return tourTravelNotesTopic;
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public void saveNewBatch(List<TourTravelNotesTopic> tourTravelNotesTopicList) {
        if (CollUtil.isNotEmpty(tourTravelNotesTopicList)) {
            tourTravelNotesTopicList.forEach(this::buildDefaultValue);
            tourTravelNotesTopicMapper.insertList(tourTravelNotesTopicList);
        }
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public boolean update(TourTravelNotesTopic tourTravelNotesTopic, TourTravelNotesTopic originalTourTravelNotesTopic) {
        MyModelUtil.fillCommonsForUpdate(tourTravelNotesTopic, originalTourTravelNotesTopic);
        // 这里重点提示，在执行主表数据更新之前，如果有哪些字段不支持修改操作，请用原有数据对象字段替换当前数据字段。
        UpdateWrapper<TourTravelNotesTopic> uw = this.createUpdateQueryForNullValue(tourTravelNotesTopic, tourTravelNotesTopic.getId());
        return tourTravelNotesTopicMapper.update(tourTravelNotesTopic, uw) == 1;
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public boolean remove(Long id) {
        return tourTravelNotesTopicMapper.deleteById(id) == 1;
    }

    @Override
    public List<TourTravelNotesTopic> getTourTravelNotesTopicList(TourTravelNotesTopic filter, String orderBy) {
        return tourTravelNotesTopicMapper.getTourTravelNotesTopicList(filter, orderBy);
    }

    @Override
    public List<TourTravelNotesTopic> getTourTravelNotesTopicListWithRelation(TourTravelNotesTopic filter, String orderBy) {
        List<TourTravelNotesTopic> resultList = tourTravelNotesTopicMapper.getTourTravelNotesTopicList(filter, orderBy);
        // 在缺省生成的代码中，如果查询结果resultList不是Page对象，说明没有分页，那么就很可能是数据导出接口调用了当前方法。
        // 为了避免一次性的大量数据关联，规避因此而造成的系统运行性能冲击，这里手动进行了分批次读取，开发者可按需修改该值。
        int batchSize = resultList instanceof Page ? 0 : 1000;
        this.buildRelationForDataList(resultList, MyRelationParam.normal(), batchSize);
        return resultList;
    }

    private TourTravelNotesTopic buildDefaultValue(TourTravelNotesTopic tourTravelNotesTopic) {
        if (tourTravelNotesTopic.getId() == null) {
            tourTravelNotesTopic.setId(idGenerator.nextLongId());
        }
        MyModelUtil.fillCommonsForInsert(tourTravelNotesTopic);
        tourTravelNotesTopic.setDataState(GlobalDeletedFlag.NORMAL);
        return tourTravelNotesTopic;
    }

    /**
     * 根据游记话题名称查询话题浏览量
     *
     * @return 查询结果集。
     */
    @Override
    public Integer getViewCountTopicByName(String name,String titleName){
        return tourTravelNotesTopicMapper.getViewCountTopicByName(name,titleName);
    }


    /**
     * 根据游记话题名称查询话题浏览量
     *
     * @return 查询结果集。
     */
    @Override
    public Map<Long, Integer> getViewCountListTopicByName(List<Long> idList){
        List<NotesViewsVo> dto = tourTravelNotesTopicMapper.getViewCountListTopicByName(idList);
        Map<Long, Integer> map=new HashMap<>();
        for (NotesViewsVo notesViewsVo : dto) {
            map.put(notesViewsVo.getId(),notesViewsVo.getTotalPageViews());
        }
        return map;
    }
    /**
     * 根据游记话题名称查询话题浏览量
     *
     * @return 查询结果集。
     */
    @Override
    public Integer getCommentCountTopicByName(String name,String titleName){
        return tourTravelNotesTopicMapper.getCommentCountTopicByName(name,titleName);
    }

    /**
     * 根据游记话题名称查询话题浏览量
     *
     * @return 查询结果集。
     */
    @Override
    public Integer getCountTravelNotesByName(String name,String titleName){
        return tourTravelNotesTopicMapper.getCountTravelNotesByName(name,titleName);
    }

    /**
     * 根据游记话题名称查询关联的游记id
     *
     * @return 查询结果集。
     */
    @Override
    public List<TourismProjectTravelNotesWrite> getTravelNotesByTopicName(String name,String orderBy){
        return tourTravelNotesTopicMapper.getTravelNotesByTopicName(name,orderBy);
    }
}
