package com.tourism.webadmin.back.service.impl;

import cn.hutool.core.collection.CollUtil;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.tourism.webadmin.back.service.*;
import com.tourism.webadmin.back.dao.*;
import com.tourism.webadmin.back.model.*;
import com.tourism.webadmin.config.DataSourceType;
import com.tourism.common.core.annotation.MyDataSource;
import com.tourism.common.core.base.dao.BaseDaoMapper;
import com.tourism.common.core.object.MyRelationParam;
import com.tourism.common.core.base.service.BaseService;
import com.tourism.common.sequence.wrapper.IdGeneratorWrapper;
import com.github.pagehelper.Page;
import lombok.extern.slf4j.Slf4j;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.*;

/**
 * 用户与评论的点赞关系表数据操作服务类。
 *
 * @author 吃饭睡觉
 * @date 2024-09-06
 */
@Slf4j
@Service("tourUserCommentLikeRelationService")
public class TourUserCommentLikeRelationServiceImpl extends BaseService<TourUserCommentLikeRelation, Long> implements TourUserCommentLikeRelationService {

    @Autowired
    private IdGeneratorWrapper idGenerator;
    @Autowired
    private TourUserCommentLikeRelationMapper tourUserCommentLikeRelationMapper;

    /**
     * 返回当前Service的主表Mapper对象。
     *
     * @return 主表Mapper对象。
     */
    @Override
    protected BaseDaoMapper<TourUserCommentLikeRelation> mapper() {
        return tourUserCommentLikeRelationMapper;
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public TourUserCommentLikeRelation saveNew(TourUserCommentLikeRelation tourUserCommentLikeRelation) {
        tourUserCommentLikeRelationMapper.insert(this.buildDefaultValue(tourUserCommentLikeRelation));
        return tourUserCommentLikeRelation;
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public void saveNewBatch(List<TourUserCommentLikeRelation> tourUserCommentLikeRelationList) {
        if (CollUtil.isNotEmpty(tourUserCommentLikeRelationList)) {
            tourUserCommentLikeRelationList.forEach(this::buildDefaultValue);
            tourUserCommentLikeRelationMapper.insertList(tourUserCommentLikeRelationList);
        }
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public boolean update(TourUserCommentLikeRelation tourUserCommentLikeRelation, TourUserCommentLikeRelation originalTourUserCommentLikeRelation) {
        // 这里重点提示，在执行主表数据更新之前，如果有哪些字段不支持修改操作，请用原有数据对象字段替换当前数据字段。
        UpdateWrapper<TourUserCommentLikeRelation> uw = this.createUpdateQueryForNullValue(tourUserCommentLikeRelation, tourUserCommentLikeRelation.getUserId());
        return tourUserCommentLikeRelationMapper.update(tourUserCommentLikeRelation, uw) == 1;
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public boolean remove(Long userId) {
        return tourUserCommentLikeRelationMapper.deleteById(userId) == 1;
    }

    @Override
    public List<TourUserCommentLikeRelation> getTourUserCommentLikeRelationList(TourUserCommentLikeRelation filter, String orderBy) {
        return tourUserCommentLikeRelationMapper.getTourUserCommentLikeRelationList(filter, orderBy);
    }

    @Override
    public List<TourUserCommentLikeRelation> getTourUserCommentLikeRelationListWithRelation(TourUserCommentLikeRelation filter, String orderBy) {
        List<TourUserCommentLikeRelation> resultList = tourUserCommentLikeRelationMapper.getTourUserCommentLikeRelationList(filter, orderBy);
        // 在缺省生成的代码中，如果查询结果resultList不是Page对象，说明没有分页，那么就很可能是数据导出接口调用了当前方法。
        // 为了避免一次性的大量数据关联，规避因此而造成的系统运行性能冲击，这里手动进行了分批次读取，开发者可按需修改该值。
        int batchSize = resultList instanceof Page ? 0 : 1000;
        this.buildRelationForDataList(resultList, MyRelationParam.normal(), batchSize);
        return resultList;
    }

    private TourUserCommentLikeRelation buildDefaultValue(TourUserCommentLikeRelation tourUserCommentLikeRelation) {
        if (tourUserCommentLikeRelation.getUserId() == null) {
            tourUserCommentLikeRelation.setUserId(idGenerator.nextLongId());
        }
        return tourUserCommentLikeRelation;
    }
}
